// 2DTestDX9.cpp

#include "2DTestDX9.h"

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
HWND hMainWnd                      = NULL;	// Handle to window
LPDIRECT3D9             pD3D       = NULL;	// Used to create the D3DDevice
LPDIRECT3DDEVICE9       pD3DDevice = NULL;	// Our rendering device
LPDIRECT3DVERTEXBUFFER9 pVB        = NULL;	// Buffer to hold vertices
float g_mR = 0.5, g_mG = 0.5, g_mB = 0.5;	// Global light settings for material

IDTSurface m_DX9LogoImage; // Surface to hold DX9logo Image
BOOL                   bOKToRender = TRUE;	// FALSE if an error occurs, will stop rendering

//-----------------------------------------------------------------------------
// The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{		
    switch (msg)
    {
		case WM_TIMER:
		case WM_DESTROY:
			PostQuitMessage(0);
			return 0;

		case WM_PAINT:
			Render();
            ValidateRect(hWnd, NULL);
            return 0;

		case WM_SIZE:
			InvalidateRect(hWnd, NULL, TRUE);
    }

    return DefWindowProc(hWnd, msg, wParam, lParam);
}


//-----------------------------------------------------------------------------
// The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain(HINSTANCE hInst, HINSTANCE, LPSTR, INT)
{
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), LoadIcon(GetModuleHandle(NULL), (LPCTSTR)IDI_DX9), LoadCursor(NULL, IDC_ARROW), NULL, NULL,
                      "2DTest", NULL };
    RegisterClassEx(&wc);

    // Create the application's window
    hMainWnd = CreateWindow("2DTest", "2D in DirectX9!", 
                              WS_OVERLAPPEDWINDOW, 100, 100, 300, 300,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL);

    // Initialize Direct3D
    if (SUCCEEDED(InitD3D(hMainWnd)))
    {
        // Show the window
        ShowWindow(hMainWnd, SW_SHOWDEFAULT);
        UpdateWindow(hMainWnd);

        // Enter the message loop
        MSG msg; 
        while (GetMessage(&msg, NULL, 0, 0))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
	else
	{
		DisplayMessage("Failed to initialize Direct3D!");
	}

    // Clean up everything and exit the app
    Cleanup();

    UnregisterClass("2DTest", wc.hInstance);

    return 0;
}


//-----------------------------------------------------------------------------
// Displays a messagebox
//-----------------------------------------------------------------------------
VOID DisplayMessage(char * message)
{
	MessageBox(hMainWnd, message, "2DTest", MB_OK);
}


//-----------------------------------------------------------------------------
// Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D(HWND hWnd)
{
	// Create the D3D object, which is needed to create the D3DDevice.
    if ((pD3D = Direct3DCreate9(D3D_SDK_VERSION)) == NULL) return E_FAIL;

    // Get the current desktop display mode
    D3DDISPLAYMODE d3ddm;
    if (FAILED( pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm))) return E_FAIL;

    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory(&d3dpp, sizeof(d3dpp));
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.Flags = D3DPRESENTFLAG_LOCKABLE_BACKBUFFER; // We want to be able to directly write to backbuffer memory

	// Create D3D Device
    if (FAILED(pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &pD3DDevice)))
    {
        return E_FAIL;
    }

	// Load image to surface
	m_DX9LogoImage.Initialize(pD3DDevice);
	int ret = m_DX9LogoImage.CreateSurfaceFromFile("dx9logo.bmp", d3ddm.Format);
	if (ret != DT_OK)
	{
		DisplayMessage("Could not load DX9logo image!");
	}

    return S_OK;
}

//-----------------------------------------------------------------------------
// Releases all initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{
    if (pD3DDevice != NULL)
	{
        pD3DDevice->Release();
		pD3DDevice = NULL;
	}

    if (pD3D != NULL)
	{
        pD3D->Release();
		pD3D = NULL;
	}
}


//-----------------------------------------------------------------------------
// Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
	// Must be initialized
    if (pD3DDevice == NULL) return;

    // Clear the backbuffer to a black
    pD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(0,0,0), 1.0f, 0);
    
	// No need to do beginscene and end scene since there's no 3D rendering going on

	if (bOKToRender)
	{
		RECT rect2; rect2.left = 0; rect2.top=0; rect2.right=256; rect2.bottom=256;
		
		// Get backbuffer and render to it
		LPDIRECT3DSURFACE9 surf;
		if (SUCCEEDED(pD3DDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &surf)))
		{

			RECT rect; rect.left = 0; rect.top=0; rect.right=256; rect.bottom=256;
			int ret = m_DX9LogoImage.BltFast(surf, 0, 0, rect);

			if (ret != DT_OK)
			{
				if (ret == DT_INVALID_FORMAT)
				{
					bOKToRender = FALSE;
					DisplayMessage("Invalid pixelformat on source or destination surface! Rendering stopped.");
				}
			}

			surf->Release();
		}
		else
		{
			DisplayMessage("Could not get the backbuffer! Rendering stopped.");
			bOKToRender = FALSE;
		}
	}

    // Present the backbuffer contents to the display
    pD3DDevice->Present(NULL, NULL, NULL, NULL);
}


